import yaml
from collections import OrderedDict


def ordered_load(stream, Loader=yaml.SafeLoader, object_pairs_hook=OrderedDict):
    class OrderedLoader(Loader):
        pass

    def construct_mapping(loader, node):
        loader.flatten_mapping(node)
        return object_pairs_hook(loader.construct_pairs(node))

    OrderedLoader.add_constructor(
        yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
        construct_mapping
    )
    return yaml.load(stream, OrderedLoader)


def ordered_dump(data, stream=None, Dumper=yaml.SafeDumper, **kwds):
    class OrderedDumper(Dumper):
        pass

    def dict_representer(dumper, data):
        return dumper.represent_mapping(
            yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
            data.items()
        )

    OrderedDumper.add_representer(OrderedDict, dict_representer)
    return yaml.dump(data, stream, OrderedDumper, **kwds)



class ChpipException(Exception):
    msg_fmt = 'An unknown exception occurred.'

    def __init__(self, *args, **kwargs):
        msg = self.msg_fmt.format(*args, **kwargs)
        super(ChpipException, self).__init__(msg)


class NoAvailableIndex(ChpipException):
    msg_fmt = 'There is no available index to change. Please use `chpip set` to set one.'


class IndexNameNotFound(ChpipException):
    msg_fmt = 'There is no index with name {name}. Please use `chpip set` to set one.'


class InvalidIndexName(ChpipException):
    msg_fmt = 'Invalid index name `{name}`. Cannot use revered name.'


class InvalidIndexURL(ChpipException):
    msg_fmt = 'Invalid base URL `{url}` for Python package index.'


class RequestError(ChpipException):
    msg_fmt = 'Request to `{url}` error. Reason: {reason}.'



import sys

import pytest

from chpip.core import PY2, ChpipManager

if PY2:
    import shutil
    import itertools
    import gc
    from weakref import ref
    import warnings
    from tempfile import mkdtemp, template


    class finalize(object):
        """Class for finalization of weakrefable objects

        finalize(obj, func, *args, **kwargs) returns a callable finalizer
        object which will be called when obj is garbage collected. The
        first time the finalizer is called it evaluates func(*arg, **kwargs)
        and returns the result. After this the finalizer is dead, and
        calling it just returns None.

        When the program exits any remaining finalizers for which the
        atexit attribute is true will be run in reverse order of creation.
        By default atexit is true.
        """

        # Finalizer objects don't have any state of their own.  They are
        # just used as keys to lookup _Info objects in the registry.  This
        # ensures that they cannot be part of a ref-cycle.

        __slots__ = ()
        _registry = {}
        _shutdown = False
        _index_iter = itertools.count()
        _dirty = False
        _registered_with_atexit = False

        class _Info(object):
            __slots__ = ("weakref", "func", "args", "kwargs", "atexit", "index")

        def __init__(*args, **kwargs):
            if len(args) >= 3:
                self, obj, func = args[:3]
                args = args[3:]
            elif not args:
                raise TypeError("descriptor '__init__' of 'finalize' object "
                                "needs an argument")
            else:
                if 'func' not in kwargs:
                    raise TypeError('finalize expected at least 2 positional '
                                    'arguments, got %d' % (len(args) - 1))
                func = kwargs.pop('func')
                if len(args) >= 2:
                    self, obj = args[:2]
                    args = args[2:]
                    import warnings
                    warnings.warn("Passing 'func' as keyword argument is deprecated",
                                  DeprecationWarning, stacklevel=2)
                else:
                    if 'obj' not in kwargs:
                        raise TypeError('finalize expected at least 2 positional '
                                        'arguments, got %d' % (len(args) - 1))
                    obj = kwargs.pop('obj')
                    self = args[0]
                    args = args[1:]
                    import warnings
                    warnings.warn("Passing 'obj' as keyword argument is deprecated",
                                  DeprecationWarning, stacklevel=2)
            args = tuple(args)

            if not self._registered_with_atexit:
                # We may register the exit function more than once because
                # of a thread race, but that is harmless
                import atexit
                atexit.register(self._exitfunc)
                finalize._registered_with_atexit = True
            info = self._Info()
            info.weakref = ref(obj, self)
            info.func = func
            info.args = args
            info.kwargs = kwargs or None
            info.atexit = True
            info.index = next(self._index_iter)
            self._registry[self] = info
            finalize._dirty = True

        __init__.__text_signature__ = '($self, obj, func, /, *args, **kwargs)'

        def __call__(self, _=None):
            """If alive then mark as dead and return func(*args, **kwargs);
            otherwise return None"""
            info = self._registry.pop(self, None)
            if info and not self._shutdown:
                return info.func(*info.args, **(info.kwargs or {}))

        def detach(self):
            """If alive then mark as dead and return (obj, func, args, kwargs);
            otherwise return None"""
            info = self._registry.get(self)
            obj = info and info.weakref()
            if obj is not None and self._registry.pop(self, None):
                return (obj, info.func, info.args, info.kwargs or {})

        def peek(self):
            """If alive then return (obj, func, args, kwargs);
            otherwise return None"""
            info = self._registry.get(self)
            obj = info and info.weakref()
            if obj is not None:
                return (obj, info.func, info.args, info.kwargs or {})

        @property
        def alive(self):
            """Whether finalizer is alive"""
            return self in self._registry

        @property
        def atexit(self):
            """Whether finalizer should be called at exit"""
            info = self._registry.get(self)
            return bool(info) and info.atexit

        @atexit.setter
        def atexit(self, value):
            info = self._registry.get(self)
            if info:
                info.atexit = bool(value)

        def __repr__(self):
            info = self._registry.get(self)
            obj = info and info.weakref()
            if obj is None:
                return '<%s object at %#x; dead>' % (type(self).__name__, id(self))
            else:
                return '<%s object at %#x; for %r at %#x>' % \
                       (type(self).__name__, id(self), type(obj).__name__, id(obj))

        @classmethod
        def _select_for_exit(cls):
            # Return live finalizers marked for exit, oldest first
            L = [(f, i) for (f, i) in cls._registry.items() if i.atexit]
            L.sort(key=lambda item: item[1].index)
            return [f for (f, i) in L]

        @classmethod
        def _exitfunc(cls):
            # At shutdown invoke finalizers for which atexit is true.
            # This is called once all other non-daemonic threads have been
            # joined.
            reenable_gc = False
            try:
                if cls._registry:
                    import gc
                    if gc.isenabled():
                        reenable_gc = True
                        gc.disable()
                    pending = None
                    while True:
                        if pending is None or finalize._dirty:
                            pending = cls._select_for_exit()
                            finalize._dirty = False
                        if not pending:
                            break
                        f = pending.pop()
                        try:
                            # gc is disabled, so (assuming no daemonic
                            # threads) the following is the only line in
                            # this function which might trigger creation
                            # of a new finalizer
                            f()
                        except Exception:
                            sys.excepthook(*sys.exc_info())
                        assert f not in cls._registry
            finally:
                # prevent any more finalizers from executing during shutdown
                finalize._shutdown = True
                if reenable_gc:
                    gc.enable()


    class TemporaryDirectory(object):
        """Create and return a temporary directory.  This has the same
        behavior as mkdtemp but can be used as a context manager.  For
        example:

            with TemporaryDirectory() as tmpdir:
                ...

        Upon exiting the context, the directory and everything contained
        in it are removed.
        """

        def __init__(self, suffix='', prefix=template, dir=None):
            self.name = mkdtemp(suffix, prefix, dir)
            self._finalizer = finalize(
                self, self._cleanup, self.name,
                warn_message="Implicitly cleaning up {!r}".format(self))

        @classmethod
        def _cleanup(cls, name, warn_message):
            shutil.rmtree(name)
            warnings.warn(warn_message, RuntimeWarning)

        def __repr__(self):
            return "<{} {!r}>".format(self.__class__.__name__, self.name)

        def __enter__(self):
            return self.name

        def __exit__(self, exc, value, tb):
            self.cleanup()

        def cleanup(self):
            if self._finalizer.detach():
                shutil.rmtree(self.name)
else:
    from tempfile import TemporaryDirectory


@pytest.fixture(scope='function')
def chpip_manager():
    with TemporaryDirectory() as tmp_dirname:
        chpip = ChpipManager(pip_dirname=tmp_dirname)
        yield chpip



import pytest

from chpip import exception
from chpip.core import DEFAULT_INDEX_NAME, DEFAULT_INDEX_URL


class TestChangeIndex(object):
    def test_set_invalid_index_name(self, chpip_manager):
        with pytest.raises(exception.InvalidIndexName):
            chpip_manager.set_index(DEFAULT_INDEX_NAME, DEFAULT_INDEX_URL)

    def test_set_invalid_index_url(self, chpip_manager):
        with pytest.raises(exception.InvalidIndexURL):
            chpip_manager.set_index('test', 'invalid')

    def test_set_index(self, chpip_manager):
        name1 = 't1'
        index_url1 = 'https://test1.com'
        chpip_manager.set_index(name1, index_url1)
        chpip_data = chpip_manager.get_chpip_data()
        assert chpip_data == {
            'indexes': {
                name1: {
                    'index_url': index_url1
                }
            }
        }

        name2 = 't2'
        index_url2 = 'https://test2.com'
        chpip_manager.set_index(name2, index_url2)
        chpip_data = chpip_manager.get_chpip_data()
        assert chpip_data == {
            'indexes': {
                name1: {
                    'index_url': index_url1
                },
                name2: {
                    'index_url': index_url2
                }
            }
        }

        index_url2_new = 'https://test2-new.com'
        chpip_manager.set_index(name2, index_url2_new)
        chpip_data = chpip_manager.get_chpip_data()
        assert chpip_data == {
            'indexes': {
                name1: {
                    'index_url': index_url1
                },
                name2: {
                    'index_url': index_url2_new
                }
            }
        }



import os
import sys
import configparser
import yaml
import requests

from chpip import exception
from chpip.utils import ordered_load, ordered_dump

PY2 = sys.version_info[0] == 2
WIN = sys.platform == 'win32'
DEFAULT_INDEX_NAME = 'default'
DEFAULT_INDEX_URL = 'https://pypi.org/simple'
SOURCE_PIP_CONFIG_URL = 'https://raw.githubusercontent.com/Prodesire/chpip/main/pip.yml'


class ChpipManager(object):

    def __init__(self, pip_dirname=None):
        if WIN:
            appdata_path = os.getenv('APPDATA')
            self.pip_dirname = os.path.join(appdata_path, 'pip') if not pip_dirname else pip_dirname
            self.pip_path = os.path.join(self.pip_dirname, 'pip.ini')
        else:
            self.pip_dirname = os.path.expanduser('~/.config/pip') if not pip_dirname else pip_dirname
            self.pip_path = os.path.join(self.pip_dirname, 'pip.conf')
        self.chpip_path = os.path.join(self.pip_dirname, '.chpip.yml')

    def _ensure_pip_dirname(self):
        if not os.path.exists(self.pip_dirname):
            os.makedirs(self.pip_dirname)

    def change_index(self, name=None):
        chpip_data = self.get_chpip_data()
        indexes = chpip_data.get('indexes')
        if not indexes:
            raise exception.NoAvailableIndex()

        if name:
            if name not in indexes:
                raise exception.IndexNameNotFound(name=name)
            next_index_url = indexes[name]['index_url']
        else:
            name = chpip_data.get('last_index_name')
            if not name:
                name = next(indexes.iterkeys() if PY2 else iter(indexes.keys()))
            next_index_url = indexes[name]['index_url']

        current_index_name = chpip_data.get('current_index_name')
        if not current_index_name:
            current_index_name = DEFAULT_INDEX_NAME
            current_index_url = DEFAULT_INDEX_URL
        else:
            current_index_url = indexes[current_index_name]['index_url']

        self._ensure_pip_dirname()
        config = self.get_pip_config()
        if not config.has_section('global'):
            config.add_section('global')
        config.set('global', 'index-url', next_index_url)
        with open(self.pip_path, 'w') as f:
            config.write(f)

        with open(self.chpip_path, 'w') as f:
            chpip_data['last_index_name'] = current_index_name
            chpip_data['current_index_name'] = name
            indexes = chpip_data['indexes']
            if current_index_name not in indexes:
                indexes[current_index_name] = {'index_url': current_index_url}
            f.write(ordered_dump(chpip_data))
        return name

    def set_index(self, name, index_url):
        if name == DEFAULT_INDEX_NAME:
            raise exception.InvalidIndexName(name=name)

        if not index_url.startswith(('http://', 'https://')):
            raise exception.InvalidIndexURL(url=index_url)

        chpip_data = self.get_chpip_data()
        self._ensure_pip_dirname()
        with open(self.chpip_path, 'w') as f:
            indexes = chpip_data.setdefault('indexes', {})
            indexes[name] = {'index_url': index_url}
            f.write(ordered_dump(chpip_data))
        return name

    def get_pip_config(self):
        config = configparser.ConfigParser()
        config.read(self.pip_path)
        return config

    def get_chpip_data(self):
        if os.path.exists(self.chpip_path):
            with open(self.chpip_path, 'r') as f:
                chpip_data = ordered_load(f.read()) or {}
        else:
            chpip_data = {}
        return chpip_data

    def show(self):
        chpip_data = self.get_chpip_data()
        if not chpip_data:
            return ''

        current_index_name = chpip_data.get('current_index_name')
        indexes = chpip_data.get('indexes') or {}
        lines = []
        for name in indexes:
            index_data = indexes[name]
            index_url = index_data.get('index_url')
            if name == current_index_name:
                line = '* {} ({})'.format(name, index_url)
            else:
                line = '  {} ({})'.format(name, index_url)
            lines.append(line)
        return '\n'.join(lines)

    def list(self):
        resp = requests.get(SOURCE_PIP_CONFIG_URL)
        if resp.status_code != 200:
            raise exception.RequestError(url=SOURCE_PIP_CONFIG_URL, reason=resp.reason)

        pip_conf = yaml.safe_load(resp.text)
        indexes = pip_conf.get('indexes') or {}
        lines = []
        for name in indexes:
            index_url = indexes[name]
            line = '  {} ({})'.format(name, index_url)
            lines.append(line)
        return '\n'.join(lines)



import sys

import click

from chpip import exception
from chpip.core import ChpipManager

pip_manager = ChpipManager()


@click.group(
    help='A tool to manage the base URL of the Python package index.',
    invoke_without_command=True)
@click.option('-n', '--name', help='Use the Python package index with the specified name.')
@click.pass_context
def cli(ctx, name=None):
    if ctx.invoked_subcommand is None:
        try:
            name = pip_manager.change_index(name)
            click.echo('Change Python package index to `{}` successful.'.format(name))
        except exception.ChpipException as e:
            click.echo(str(e))
            ctx.exit(1)


@cli.command(help='Set the base URL of the Python package index with name.')
@click.option('-n', '--name', required=True, help='Name of the Python package index.')
@click.option('-i', '--index-url',
              required=True,
              help='Base URL of the Python Package Index. '
                   'This should point to a repository compliant with PEP 503 (the simple repository API) '
                   'or a local directory laid out in the same format.')
@click.pass_context
def set(ctx, name, index_url):
    try:
        name = pip_manager.set_index(name, index_url)
        click.echo('Set Python package index with name `{}` successful.'.format(name))
    except exception.ChpipException as e:
        click.echo(str(e))
        ctx.exit(1)


@cli.command(help='Show all base URLs of the Python package index. '
                  'Current index is marked with *.')
@click.pass_context
def show(ctx):
    try:
        message = pip_manager.show()
        click.echo(message)
    except exception.ChpipException as e:
        click.echo(str(e))
        ctx.exit(1)


@cli.command(help='List the commonly used URLs of the Python package index, '
                  'which can be set by executing the `chpip set` command.')
@click.pass_context
def list(ctx):
    try:
        message = pip_manager.list()
        click.echo(message)
    except exception.ChpipException as e:
        click.echo(str(e))
        ctx.exit(1)


if __name__ == '__main__':
    cli()



import sys
from chpip import __version__
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

PY2 = sys.version_info[0] == 2


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', 'Arguments to pass into py.test')]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        try:
            from multiprocessing import cpu_count
            self.pytest_args = ['-n', str(cpu_count())]
        except (ImportError, NotImplementedError):
            self.pytest_args = ['-n', '1']

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest

        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


test_requirements = []
for line in open('requirements-dev.txt'):
    requirement = line.strip()
    if requirement:
        test_requirements.append(requirement)

open_kwargs = {} if PY2 else {'encoding': 'utf-8'}
setup(
    name='chpip',
    version=__version__,
    description='A tool to manage the base URL of the Python package index.',
    long_description=open('README.md', **open_kwargs).read(),
    long_description_content_type='text/markdown',
    author='Prodesire',
    author_email='wangbinxin001@126.com',

    se='MIT License',
    url='https://github.com/Prodesire/chpip',
    cmdclass={'test': PyTest},
    tests_require=test_requirements,
    install_requires=[
        'click>=0.7.0',
        'pyyaml>=5.0.0,<7.0.0',
        'requests',
    ],
    packages=find_packages(),
    entry_points={
        'console_scripts': ['chpip=chpip.__main__:cli']
    },
    classifiers=[
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Software Development :: Libraries'
    ],
)



import pytest

from chpip import exception
from chpip.core import DEFAULT_INDEX_NAME, DEFAULT_INDEX_URL


class TestChangeIndex(object):
    TEST_INDEX_NAME = 'test'
    TEST_INDEX_URL = 'https://test.com'

    def test_change_no_available_indexes(self, chpip_manager):
        with pytest.raises(exception.NoAvailableIndex):
            chpip_manager.change_index()

    def test_change_index_not_found(self, chpip_manager):
        chpip_manager.set_index(self.TEST_INDEX_NAME, self.TEST_INDEX_URL)
        with pytest.raises(exception.IndexNameNotFound):
            chpip_manager.change_index('not_found')

    def test_change_index_without_name(self, chpip_manager):
        chpip_manager.set_index(self.TEST_INDEX_NAME, self.TEST_INDEX_URL)
        name = chpip_manager.change_index()
        assert name == self.TEST_INDEX_NAME
        pip_conf = chpip_manager.get_pip_config()
        assert pip_conf['global']['index-url'] == self.TEST_INDEX_URL

        name = chpip_manager.change_index()
        assert name == DEFAULT_INDEX_NAME
        pip_conf = chpip_manager.get_pip_config()
        assert pip_conf['global']['index-url'] == DEFAULT_INDEX_URL

        name = chpip_manager.change_index()
        assert name == self.TEST_INDEX_NAME
        pip_conf = chpip_manager.get_pip_config()
        assert pip_conf['global']['index-url'] == self.TEST_INDEX_URL

    def test_change_index_with_name(self, chpip_manager):
        chpip_manager.set_index(self.TEST_INDEX_NAME, self.TEST_INDEX_URL)
        name = chpip_manager.change_index(self.TEST_INDEX_NAME)
        assert name == self.TEST_INDEX_NAME
        pip_conf = chpip_manager.get_pip_config()
        assert pip_conf['global']['index-url'] == self.TEST_INDEX_URL

        name = chpip_manager.change_index(DEFAULT_INDEX_NAME)
        assert name == DEFAULT_INDEX_NAME
        pip_conf = chpip_manager.get_pip_config()
        assert pip_conf['global']['index-url'] == DEFAULT_INDEX_URL

        name = chpip_manager.change_index(DEFAULT_INDEX_NAME)
        assert name == DEFAULT_INDEX_NAME
        pip_conf = chpip_manager.get_pip_config()
        assert pip_conf['global']['index-url'] == DEFAULT_INDEX_URL
